@@ -19,7 +19,7 @@ namespace League\Csv;
 class CannotInsertRecord extends Exception
 {
     /** The record submitted for insertion. */
-    protected array $record;
+    protected array $record = [];
     /** Validator which did not validate the data. */
     protected string $name = '';
 
