@@ -41,7 +41,7 @@ final class ReaderTest extends TabularDataReaderTestCase
         $tmp = new SplTempFileObject();
         $tmp->setCsvControl(escape: '\\');
         foreach ($this->expected as $row) {
-            $tmp->fputcsv($row);
+            $tmp->fputcsv($row, escape: '\\');
         }
 
         $this->csv = Reader::createFromFileObject($tmp);
@@ -52,7 +52,7 @@ final class ReaderTest extends TabularDataReaderTestCase
         unset($this->csv);
     }
 
-    protected function tabularData(): TabularDataReader
+    protected function tabularDataWithoutHeader(): TabularDataReader
     {
         $csv = <<<CSV
 date,temperature,place
@@ -160,15 +160,15 @@ EOF;
         $file = new SplTempFileObject();
         $file->setCsvControl(escape: '\\');
         foreach ($raw as $row) {
-            $file->fputcsv($row);
+            $file->fputcsv($row, escape: '\\');
         }
         $csv = Reader::createFromFileObject($file);
         $csv->setHeaderOffset(0);
 
-        $res = Statement::create()->process($csv);
+        $res = (new Statement())->process($csv);
         self::assertEquals($csv->nth(3), $res->nth(3));
-        self::assertEquals($csv->fetchColumnByName('firstname'), $res->fetchColumnByName('firstname'));
-        self::assertEquals($csv->fetchColumnByOffset(1), $res->fetchColumnByOffset(1));
+        self::assertEquals($csv->fetchColumn('firstname'), $res->fetchColumn('firstname'));
+        self::assertEquals($csv->fetchColumn(1), $res->fetchColumn(1));
         self::assertEquals($csv->fetchPairs('lastname', 0), $res->fetchPairs('lastname', 0));
     }
 
@@ -368,7 +368,7 @@ EOF;
         $tmp = new SplTempFileObject();
         $tmp->setCsvControl(escape: '\\');
         foreach ($expected as $row) {
-            $tmp->fputcsv($row);
+            $tmp->fputcsv($row, escape: '\\');
         }
 
         $reader = Reader::createFromFileObject($tmp)->setHeaderOffset(0);
@@ -647,7 +647,7 @@ CSV;
         fputcsv($fp, ['abc', '123'], escape: '');
 
         $csv = Reader::createFromStream($fp);
-        $csv->addStreamFilter('convert.iconv.UTF-8/UTF-16');
+        $csv->appendStreamFilterOnRead('convert.iconv.UTF-8/UTF-16');
 
         // An explicitly closed file handle makes the stream filter resources invalid
         fclose($fp);
